/*
 * Copyright 2013 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "Constants.js" as Const

FocusScope {
    id: noteItem

    property string dateText
    property string noteText

    onNoteTextChanged: noteContent.setData(noteText);

    property alias dirty: noteContent.dirty
    property real collapsedHeight
    property real expandedHeight
    property bool isCurrentItem: false
    property bool isExpanded: false

    signal noteDeleted
    signal activated
    signal deactivated

    /* WORKAROUND for https://bugreports.qt-project.org/browse/QTBUG-30137
       Expand/contract immediately the note to trigger the move transition on
       the Column containing this item. Since the inner noteContent.height is bound
       to noteItem.height and has a Behavior on it with the same duration of the
       Column.move transition, they will animate in sync. */
    height: isExpanded ? Math.max(noteContent.contentHeight, expandedHeight) : collapsedHeight

    onIsExpandedChanged: {
        if (!isExpanded) collapsedContent.fill(noteContent.model);
        else if (readNoteData().length === 0) noteContent.focus = true;
    }

    Component.onCompleted: removableMenu.itemRemoved.connect(noteDeleted)

    function readNoteData() { return noteContent.readData() }

    // embed into RemovableMenu to handle both left and right swipe
    RemovableMenu {
        id: removableMenu
        removable: !noteItem.isExpanded

        anchors {
            fill: parent
            leftMargin: units.gu(1)
            rightMargin: units.gu(1)
        }

        MouseArea {
            anchors.fill: parent
            enabled: !removableMenu.removable
            onPressAndHold: {
                if (noteContent.activeTextAreaIndex != -1) {
                    noteContent.activateClipboard("text", noteContent.activeTextArea,
                                                  noteContent.activeTextAreaIndex)
                }
            }

            onClicked: noteContent.focus = true;
        }

        BorderImage {
            anchors.fill: noteContent
            anchors.margins: units.gu(0.5) * -1
            source: "graphics/note_bg.sci"
        }

        MixedEdit {
            id: noteContent
            width: parent.width
            height: noteItem.height

            opacity: noteItem.isExpanded ? 1.0 : 0.0
            onModelUpdated: collapsedContent.fill(noteContent.model);

            onActivated: noteItem.activated()
            onActiveFocusChanged: if (!activeFocus) focus = false;

            Behavior on height {
                NumberAnimation { duration: Const.animationDuration }
            }
        }

        CollapsedNote {
            id: collapsedContent
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top: parent.top
            height: noteItem.collapsedHeight
            opacity: noteItem.isExpanded ? 0.0 : 1.0
        }

        CollapsedNoteDecorations {
            anchors.bottom: noteContent.bottom
            anchors.left: noteContent.left
            anchors.right: noteContent.right
            date: noteItem.dateText
            opacity: collapsedContent.opacity
        }

        onClicked: noteItem.activated()
    }

    InverseMouseArea {
        anchors.fill: parent
        property int pressedAtY: 0
        onPressed: pressedAtY = mouse.y
        onReleased: if (pressedAtY == mouse.y) noteItem.deactivated()
        enabled: noteItem.isExpanded
        propagateComposedEvents: true
    }
}
