/*
 * Copyright 2013 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1

/*!
    \qmltype RemovableMenu
    \inqmlmodule Ubuntu.ChewieUI 0.1
    \brief Allows remove item from a ListView by dragging it to right or left

    Example:
    \qml
        import QtQuick 2.0
        import Ubuntu.ChewieUI 0.1 as ChewieUI

        Item {
            height: 720
            width: 480

            ListModel {
                id: myModel

                ListElement {
                    name: "Bill Smith"
                }
                ListElement {
                    name: "John Brown"
                }
                ListElement {
                    name: "Sam Wise"
                }
            }

            ListView {
                id: view

                anchors.fill: parent
                model: myModel

                delegate: ChewieUI.RemovableMenu {
                    height: 50
                    width: view.width
                    listView: view
                    onSwipingChanged: view.interactive = !swiping
                    Text {
                        text: name
                        anchors.fill: parent
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignVCenter
                    }
                }
            }
        }
    \endqml

    \b{This component is under heavy development.}
*/

Item {
    id: __dragItem

    /*!
      \preliminary
      Enable or disble this item
     */
    property alias removable: __dragArea.visible

    /*!
      \preliminary
      This is a read-only property that notify when the item is swiping
     */
    property bool swiping: false

    default property alias content: __contents.children

    /*!
      \preliminary
      This handler is called when the item is removed from the list
     */
    signal itemRemoved

    signal clicked

    Component.onCompleted: __dragArea.clicked.connect(clicked)

    SequentialAnimation {
        id: __removeItem
        NumberAnimation {
            target: __dragItem
            property: "implicitHeight"
            to: 0
            duration: 200
        }
        ScriptAction {
             script: itemRemoved()
        }
    }

    Item {
        id: __backgroundItem

        anchors.top: parent.top
        anchors.bottom: parent.bottom

        Label {
            id: __backgroundText

            property int slidingMargin: units.gu(3)

            anchors {
                fill: parent
                leftMargin: units.gu(2)
            }
            verticalAlignment: Text.AlignVCenter
            text: "Clear"
            font.weight: Font.Bold
            color: "#322718"
            visible: false
        }

        states: [
            State {
                name: "SLIDING-RIGHT"
                AnchorChanges {
                    target: __backgroundItem
                    anchors.left: parent.left
                    anchors.right: __contents.left
                }
                PropertyChanges {
                    target: __backgroundText
                    anchors.rightMargin: slidingMargin
                    anchors.leftMargin: 0
                    horizontalAlignment: Text.AlignRight
                    visible: true
                }
                PropertyChanges {
                    target: __backgroundItem
                }
            },
            State {
                name: "SLIDING-LEFT"
                AnchorChanges {
                    target: __backgroundItem
                    anchors.left: __contents.right
                    anchors.right: parent.right
                }
                PropertyChanges {
                    target: __backgroundText
                    anchors.rightMargin: 0
                    anchors.leftMargin: slidingMargin
                    horizontalAlignment: Text.AlignLeft
                    visible: true
                }
                PropertyChanges {
                    target: __backgroundItem
                }
            }
        ]
    }

    Item {
        id: __contents
        width: parent.width
        height: parent.height

        Behavior on x {
            SequentialAnimation {
                NumberAnimation {
                    duration: 200
                }
                ScriptAction {
                     script: {
                         __dragArea.commitDrag()
                    }
                }
            }
        }

        onXChanged: {
            if (x > 0) {
                __backgroundItem.state = "SLIDING-RIGHT"
            } else {
                __backgroundItem.state = "SLIDING-LEFT"
            }
        }

        MouseArea {
            id: __dragArea

            property int mouseMoveOffset: units.gu(4)
            property int itemMoveOffset: width * 0.3
            property int pressedPosition: 0
            property int positionEnded: 0
            property bool held: false
            property bool removeItem: false

//            propagateComposedEvents: true
            anchors.fill: parent
            drag.axis: Drag.XAxis

            // make sure this is over any other mouse area
            z: 10

            onPressed: {
                pressedPosition = mouse.x
            }

            onMouseXChanged: {
                var mouseOffset = pressedPosition - mouse.x
                if ((pressedPosition != -1) && !held && ( Math.abs(mouseOffset) >= mouseMoveOffset)) {
                    startDrag();
                }
            }

            onPositionChanged: {
                if (held) {
                    positionEnded = __contents.x;
                }
                mouse.accepted = false
            }

            onReleased: {
                if (held) {
                    endDrag();
                }
                mouse.accepted = false
            }

            onDoubleClicked: mouse.accepted = false
            onPressAndHold: mouse.accepted = false
            onWheel: wheel.accepted = false

            // notify the start of the drag operation
            function startDrag() {
                __dragItem.swiping = true
                __contents.anchors.left = undefined
                __dragArea.drag.target = __contents

                held = true
                drag.maximumX = __dragItem.parent.width
                drag.minimumX = (width * -1)
                __backgroundItem.visible = true
            }

            // Commit the necessary changes to remove or not the item based on the mouse final position
            function commitDrag() {
                pressedPosition = -1
                __dragArea.drag.target = null
                held = false
                positionEnded = __contents.x

                if (removeItem) {
                    __removeItem.start()
                }
                removeItem = false

                __dragItem.swiping = false
                __backgroundItem.state = ""
            }

            // notify the releaso of the mouse button and the end of the drag operation
            function endDrag() {
                if (Math.abs(__contents.x) < itemMoveOffset && held == true) {
                    removeItem = false
                    if (__contents.x == 0) {
                        commitDrag()
                    } else {
                        __contents.x = 0;
                    }
                } else if (held == true) {
                    removeItem = true
                    if (__contents.x > 0) {
                        __contents.x = __contents.width
                    } else {
                        __contents.x = __contents.width * -1
                    }
                }
            }
        }
    }
}

